unit TEST_QueryUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IBSQL, IBDatabase, SysHostUnit, SysLaunchPack, SysDBWorks,
  DB, SimpleSQLQueryUnit, VirtualTrees, ExtCtrls, CustomNodeDataUnit, TestClassUnit,
  ComCtrls, JclStrings, SysFuncs, JclHookExcept, JclDebug, GDIPAPI, GDIPOBJ;

type
  TForm1 = class(TForm)
    Button1: TButton;
    MainDB: TIBDatabase;
    Panel1: TPanel;
    VT: TVirtualDrawTree;
    Panel2: TPanel;
    ProgressBar1: TProgressBar;
    Panel3: TPanel;
    Memo1: TMemo;
    CheckBox1: TCheckBox;
    Label1: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure VTDrawNode(Sender: TBaseVirtualTree;
      const PaintInfo: TVTPaintInfo);
    procedure Button2Click(Sender: TObject);
    procedure VTHotChange(Sender: TBaseVirtualTree; OldNode,
      NewNode: PVirtualNode);
    procedure VTMeasureItem(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; var NodeHeight: Integer);
    procedure VTResize(Sender: TObject);
  private
    function GetCountDocs: Integer;
    { Private declarations }
  protected
    LP: TFullLaunchPack;
    MainDir: string;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Types;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  //   TNodeDataEventHandler 
  //       

  TNodeDataEventHandler.InitEvents(VT);

  MainDir := ExtractFilePath(Application.ExeName);
  MainDB.DatabaseName := GetHostString(MainDir + '\dbhost.ini') + 'MainDB.FDB';
  MainDB.Params.Add('user_name=SYSDBA');
  MainDB.Params.Add('password=masterkey');
  MainDB.Open;
end;

function TForm1.GetCountDocs: Integer;
var
  SQLQuery: TSimpleSQLQuery;
begin
  SQLQuery := TSimpleSQLQuery.Create(MainDB);
  SQLQuery.SQL.CommaText := 'SELECT COUNT(LINK) FROM DOCS';
  SQLQuery.Prepare;
  SQLQuery.ExecQuery;
  Result := SQLQuery.FieldByName('COUNT').AsInteger;
  SQLQuery.Close;
  SQLQuery.SQL.Clear;
  SQLQuery.Free;
end;

function StrFix(const S: string): string;
var
  I: Integer;
begin
  Result := S;
  for I := 1 to Length(S) do
    if (Result[I] = #10) and (I > 1) and not(Result[I-1] = #13) then
      Insert(#13, Result, i);
  Result := Trim(Result);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  SQLQuery: TSimpleSQLQuery;
  A,B,C : Int64;
  str: string;
  TC: TTestClass;
begin
  ProgressBar1.Visible := True;
  VT.Clear;
  VT.BeginUpdate;
  Memo1.Hide;

  QueryPerformanceFrequency(C);
  QueryPerformanceCounter(A);

  ProgressBar1.Max := GetCountDocs;
  ProgressBar1.Position := 0;

  SQLQuery := TSimpleSQLQuery.Create(MainDB);
  SQLQuery.SQL.CommaText := 'SELECT FIRST :COUNT LINK FROM DOCS';
  SQLQuery.ParamByName('COUNT').AsInteger := 10;
  SQLQuery.ExecQuery;
  while not SQLQuery.Eof do
  begin
    str := SQLQuery.Current.ByName('LINK').AsTrimString;

    //      VT.RootNode
    TC := TTestClass.Create(VT, VT.RootNode);
    TC.Text := str;
    Exclude(TC.Node.States, vsHeightMeasured);
    VT.MeasureItemHeight(VT.Canvas, TC.Node);

    SQLQuery.Next;

    ProgressBar1.StepBy(1);
  end;
  SQLQuery.Close;
  SQLQuery.SQL.Clear;
  SQLQuery.Free;
  QueryPerformanceCounter(B);
  str := '  ' + FloatToStrF(1000 * (B - A) / C,ffFixed, 4, 4) + ' ms';
  Panel1.Caption := str;
  VT.EndUpdate;
  ProgressBar1.Visible := False;
  Memo1.Show;
end;

procedure TForm1.VTDrawNode(Sender: TBaseVirtualTree;
  const PaintInfo: TVTPaintInfo);
var
  i, Lines: Integer;
  r: TRect;
  TC: TTestClass;

  gpFont: TGPFont;
  gpRect: TGPRectF;
  gpFormat: TGPStringFormat;
  gpPoint: TGPPointF;
  gpCanvas, gpTmpCanvas: TGPGraphics;
  gpTmpBitmap: TGPBitmap;
  gpSolidBrush: TGPBrush;
begin
  if not Assigned(PaintInfo.Node) then Exit;

  PaintInfo.Canvas.Brush.Color := clWhite;
  PaintInfo.Canvas.FillRect(PaintInfo.CellRect);

  TC := TTestClass(TTestClass.GetNodeData(PaintInfo.Node, VT));

  gpCanvas := TGPGraphics.Create(PaintInfo.Canvas.Handle);
  gpCanvas.SetTextRenderingHint( TextRenderingHintClearTypeGridFit );
  gpCanvas.Clear(aclWhite);

  gpTmpBitmap := TGPBitmap.Create(PaintInfo.ContentRect.Right-PaintInfo.ContentRect.Left,
                               PaintInfo.ContentRect.Bottom-PaintInfo.ContentRect.Top);
  gpTmpCanvas := TGPGraphics.Create(gpTmpBitmap);
  gpTmpCanvas.SetCompositingMode( CompositingModeSourceOver );
  gpTmpCanvas.SetCompositingQuality( CompositingQualityHighSpeed );
  gpTmpCanvas.SetInterpolationMode( InterpolationModeHighQuality );
  gpTmpCanvas.SetPixelOffsetMode( PixelOffsetModeHighSpeed );
  gpTmpCanvas.SetSmoothingMode( SmoothingModeNone );

  gpTmpCanvas.Clear(aclWhite);

  gpSolidBrush := TGPSolidBrush.Create(aclBlack);
  gpFont := TGPFont.Create(Sender.Font.Name, Sender.Font.Size);
  gpFormat := TGPStringFormat.Create;

  if Sender.Selected[PaintInfo.Node] then
  begin
    gpTmpCanvas.Clear($FF00FF00);
  end;

  gpPoint.X := 0.0;
  gpPoint.Y := 0.0;
  gpRect.X := 0.0;
  gpRect.Y := 0.0;
  gpRect.Width := PaintInfo.ContentRect.Right-PaintInfo.ContentRect.Left;
  gpRect.Height := PaintInfo.ContentRect.Bottom-PaintInfo.ContentRect.Top;
  gpTmpCanvas.SetTextRenderingHint(TextRenderingHintClearTypeGridFit);
  gpFormat.SetLineAlignment(StringAlignmentCenter);

  gpTmpCanvas.DrawString(WideString(TC.Text), Length(TC.Text), gpFont, gpRect, gpFormat, gpSolidBrush);
  

  gpPoint.X := PaintInfo.ContentRect.Left;
  gpPoint.Y := PaintInfo.ContentRect.Top;
  gpCanvas.DrawImage(gpTmpBitmap, gpPoint);
  gpTmpCanvas.Free;
  gpTmpBitmap.Free;
  gpCanvas.Free;
  gpFormat.Free;
  gpFont.Free;
  gpSolidBrush.Free;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  A,B,C : Int64;
begin
  Memo1.Text := 'fox' + #10 + ' ' + 'fix';
  QueryPerformanceFrequency(C);
  QueryPerformanceCounter(A);
  Memo1.Text := StrFix(Memo1.Text);
  QueryPerformanceCounter(B);
  Memo1.Lines.Add('  ' + FloatToStrF(1000 * (B - A) / C,ffFixed, 4, 4) + ' ms');
//  Memo1.Text := '';
end;

procedure TForm1.VTHotChange(Sender: TBaseVirtualTree; OldNode,
  NewNode: PVirtualNode);
var
  i: Integer;
  A,B,C : Int64;
  TC: TTestClass;
  str: string;
  Words: TSysStringWords;
  WordsList: TStringList;
begin
  if Assigned(NewNode) then
  begin
    Memo1.Lines.Clear;
    TC := TTestClass(TTestClass.GetNodeData(NewNode, VT));
    WordsList := TStringList.Create;
    str := TC.Text;
    QueryPerformanceFrequency(C);
    QueryPerformanceCounter(A);
    if CheckBox1.Checked then
    begin
      StrFix(str);
      StrTokens(str, WordsList);
      QueryPerformanceCounter(B);
      for i := 0 to WordsList.Count-1 do
        Memo1.Lines.Add(WordsList.Strings[i]);
    end
    else
    begin
      str := StringReplace(str, #13#10, #10, [rfReplaceAll]);
      str := Trim(StringReplace(str, #10, #13#10, [rfReplaceAll]));
      GetWords(str, Words);
      QueryPerformanceCounter(B);
      for i := 0 to Length(Words)-1 do
        Memo1.Lines.Add(Words[i]);
    end;
    label1.Caption := '  ' + FloatToStrF(1000 * (B - A) / C,ffFixed, 4, 4) + ' Msg';
    WordsList.Free;
  end;
end;

procedure TForm1.VTMeasureItem(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; var NodeHeight: Integer);
var
  TC: TTestClass;
  R: TRect;
begin
  if Assigned(Node) then
  begin
    TC := TTestClass(TTestClass.GetNodeData(Node, VT));
    R := VT.GetDisplayRect(Node,NoColumn, false);
    TC.RefrHeight(R.Right-R.Left);
    NodeHeight := TC.Height;
  end;
end;

procedure TForm1.VTResize(Sender: TObject);
var
  tmpNode: PVirtualNode;
begin
  tmpNode := VT.GetFirstVisible;
  while Assigned(tmpNode) do
  begin
    Exclude(tmpNode.States, vsHeightMeasured);
    VT.MeasureItemHeight(VT.Canvas, tmpNode);
    tmpNode := VT.GetNextVisible(tmpNode);
  end;
end;

procedure AnyExceptionNotify(ExceptObj: TObject; ExceptAddr: Pointer; OSException: Boolean);
var
  Str: TStringList;
begin
  Str := TStringList.Create;
  with Form1 do 
  begin
    Memo1.Lines.BeginUpdate;
    JclLastExceptStackListToStrings(Str, false, True, True);
    Str.Insert(0, '');
    Application.MessageBox(PChar(Str.Text), '', MB_OK or MB_ICONSTOP);
    Memo1.Lines.EndUpdate; 
  end;
end;

initialization
  Include(JclStackTrackingOptions, stRawMode);
  JclStartExceptionTracking;
  JclAddExceptNotifier(AnyExceptionNotify);

finalization
  JclStopExceptionTracking;

end.
