unit TestClassUnit;

interface

uses
  Windows, Classes, Math, VirtualTrees, CustomNodeDataUnit, SysUtils, JclStrings, GDIPAPI, GDIPOBJ;
type
  PNodeText = ^TNodeText;
  TNodeText = record
    Text: string;
    Width: Integer;
  end;

  TNodeTextList=class(TList)
  private
    function Get(Index: Integer): PNodeText;
  public
    procedure Clear; override;
    function Add(inText: string; inWidth: Integer): Integer;
    property Items[Index: Integer]: PNodeText read Get; default;
  end;

type
  TTestClass = class(TCustomNodeData)
  private
    FText: string;
    FWordList: TNodeTextList;
    FHeight: Integer;
    FWidth: Integer;
    function StrFix(const S: string): string;
    procedure SetText(const inText: string);
    procedure GetTextSize(const inText: string; inWordWrap: Boolean; var inWidth, inHeight: Integer);
    function GetString(Index: Integer): string;
    function GetWidth(Index: Integer): Integer;
    function GetCount: Integer;
  public
    constructor Create(ATree: TBaseVirtualTree; AParent: PVirtualNode); override;
    destructor Destroy; override;
    procedure RefrHeight(inWidth: Integer);
    property Text: string read FText write SetText;
    property Strings[Index: Integer]: string read GetString;
    property Widths[Index: Integer]: Integer read GetWidth;
    property Count: Integer read GetCount;
    property Height: Integer read FHeight;
    property Width: Integer read FWidth;
    property Node: PVirtualNode read FNode;
  end;

implementation

{ TNodeTextList }

function TNodeTextList.Add(inText: string; inWidth: Integer): Integer;
var
  PtrNodeText: PNodeText;
begin
  New(PtrNodeText);
  PtrNodeText^.Text := inText;
  PtrNodeText^.Width := inWidth;
  Result := inherited Add(PtrNodeText);
end;

procedure TNodeTextList.Clear;
var
  i: Integer;
  Ptr: Pointer;
begin
  for i := 0 to Count - 1 do
  begin
    Dispose(PNodeText(Items[i]));
  end;
  inherited;
end;

function TNodeTextList.Get(Index: Integer): PNodeText;
begin
  Result := PNodeText(inherited Get(Index));
end;

{ TTestClass }

constructor TTestClass.Create(ATree: TBaseVirtualTree; AParent: PVirtualNode);
begin
  inherited;
  FWordList := TNodeTextList.Create;
end;

destructor TTestClass.Destroy;
begin
  FreeAndNil(FWordList);
  inherited;
end;

function TTestClass.StrFix(const S: string): string;
var
  I: Integer;
begin
  Result := S;
  for I := 1 to Length(S) do
    if (Result[I] = #10) and (I > 1) and not(Result[I-1] = #13) then
      Insert(#13, Result, i);
  Result := Trim(Result);
end;

procedure TTestClass.GetTextSize(const inText: string; inWordWrap: Boolean; var inWidth, inHeight: Integer);
var
  gpFont: TGPFont;
  outRect: TGPRectF;
  gpFormat: TGPStringFormat;
  gpPoint: TGPPointF;
  gpCanvas: TGPGraphics;
  gpBitmap: TGPBitmap;
begin
  gpBitmap := TGPBitmap.Create(FTree.Width, FTree.Height);
  try
    gpCanvas := TGPGraphics.Create(gpBitmap);
    try
      gpFont := TGPFont.Create(FTree.Font.Name, FTree.Font.Size);
      try
        gpFormat := TGPStringFormat.Create;
        try
          gpPoint.X := 0.0;
          gpPoint.Y := 0.0;
          outRect.X := 0.0;
          outRect.Y := 0.0;
          outRect.Width := 0.0;
          outRect.Height := 0.0;
          gpCanvas.SetTextRenderingHint(TextRenderingHintClearTypeGridFit);
          if not inWordWrap then
          begin
            gpCanvas.MeasureString(WideString(inText), -1, gpFont, gpPoint, gpFormat, outRect);
          end
          else
          begin
            outRect.Width := inWidth;
            outRect.Height := 1;
            gpFormat.SetFormatFlags(StringFormatFlagsNoWrap or StringFormatFlagsNoClip);
            gpCanvas.MeasureString(WideString(inText), -1, gpFont, gpPoint, gpFormat, outRect);
          end;
        finally
          gpFormat.Free;
        end;
      finally
        gpFont.Free;
      end;
    finally
      gpCanvas.Free;
    end;
  finally
    gpBitmap.Free;
  end;
  inWidth := Round(outRect.Width);
  inHeight := Round(outRect.Height);
end;

procedure TTestClass.SetText(const inText: string);
var
  i, Width, Height: Integer;
  tmpStrList: TStringList;
  tmpStr: string;
begin
  tmpStrList := TStringList.Create;
  try
    FWordList.Clear;
    FText := inText;
    StrFix(FText);
    StrTokens(FText, tmpStrList);
    for i := 0 to tmpStrList.Count-1 do
    begin
      tmpStr := tmpStrList.Strings[i];
      Width := 0;
      Height := 0;
      GetTextSize(tmpStr, False, Width, Height);
      if Width <> 0 then
      begin
        FWordList.Add(tmpStr, Width);
      end;
    end;
  finally
    tmpStrList.Free;
  end;
end;

function TTestClass.GetString(Index: Integer): string;
begin
  Result := FWordList[Index].Text;
end;

function TTestClass.GetWidth(Index: Integer): Integer;
begin
  Result := FWordList[Index].Width;
end;

function TTestClass.GetCount: Integer;
begin
  Result := FWordList.Count;
end;

procedure TTestClass.RefrHeight(inWidth: Integer);
var
  Width, Height: Integer;
begin
  Width := inWidth;
  GetTextSize(FText, True, Width, Height);
  Height := Height*((Width div inWidth)+1);
  FHeight := Height;
end;

end.
